@@ -8,7 +8,7 @@ def main():
     # Create a new viset
     print 'Creating vision dataset for Caltech101 dataset'
     dbfile = Caltech101().export()
-    db = Viset(dbfile, verbose=True)
+    db = Viset(dbfile, verbose=False)
     print 'Displaying dataset details'
     print db
     
@@ -29,26 +29,24 @@ def main():
 
     # Show a random training/testing split
     print 'Show every 500th image and annotation from a random training and testing split in "' + dbfile + '"'
-    split = db.annotation.categorization(step=500, strategy='train-test', randomize=True)
-    for (im, annotation) in split.train:
+    (train, test) = db.annotation.categorization.split(step=500, strategy='train-test', randomize=True)
+    for (im, annotation) in train:
         print 'Training image=' + str(annotation['id_img']) + ' Category=' + annotation['category']
         imshow(im)
-    for (im, annotation) in split.test:
+    for (im, annotation) in test:
         print 'Testing image=' + str(annotation['id_img']) + ' Category=' + annotation['category']
         imshow(im)
 
     # kfold splits
-    foldindex = 1
     print 'Show every 1000th image and annotation from a kfold split in "' + dbfile + '"'      
-    for fold in db.annotation.categorization(step=1000, strategy='kfold', folds=3):
-        print 'Fold # ' + str(foldindex)
+    for (k, fold) in enumerate(db.annotation.categorization.split(step=1000, strategy='kfold', folds=3)):
+        print 'Fold # ' + str(k)
         for (im, annotation) in fold.train:
             print 'Training image=' + str(annotation['id_img']) + ' Category=' + annotation['category']
             imshow(im)
         for (im, annotation) in fold.test:
             print 'Testing image=' + str(annotation['id_img']) + ' Category=' + annotation['category']
             imshow(im)
-        foldindex += 1
       
     # Cleanup cached data (optional)
     # del(db)
