@@ -70,23 +70,14 @@ def test_split():
 
     # Show all images with a step
     print '[test_viset.test_split]: show all images with step'          
-    for im in db.image(step=1):
+    for im in db.image(step=2):
         imshow(im)
 
     # Show all annotations
     print '[test_viset.test_split]: show all annotations with step'              
-    for (im, annotation) in db.annotation.categorization(step=1):
+    for (im, annotation) in db.annotation.categorization(step=2):
         print 'Image=' + str(annotation['id_img']) + ', Category=' + annotation['category']
         imshow(im)
-
-    # Show a random training/testing split
-    print '[test_viset.test_split]: training/testing split'
-    for split in db.annotation.categorization(step=1, strategy='train-test', randomize=False):
-        for (im, annotation) in split.train:
-            print 'Training image=' + str(annotation['id_img']) + ', Category=' + annotation['category']
-            imshow(im)
-        for (im, annotation) in split.test:
-            print 'Testing image=' + str(annotation['id_img']) + ', Category=' + annotation['category']
     
     # Non iterated splits
     print '[test_viset.test_split]: random training/testing split'    
