@@ -43,6 +43,9 @@ class Cache():
     def __str__(self):
         return str('this is a string')
 
+    def __getitem__(self, url):
+        return self.get(url)
+
     def _url_fragment_options(self, url, sha1=None):
         default_opts = {'sha1':sha1, 'item':None, 'reader':None, 'id':None}
         url_scheme = urlparse.urlparse(url)[0]
@@ -57,13 +60,18 @@ class Cache():
         return opt
   
     def read(self, filename, reader=None, id=None):
-        quietprint('[viset.cache][HIT]: Reading ' + reader + ' encoded image ' + str(id) + ' from file "' + filename + '"', self._verbose)
-        if reader == 'universal':
+        """Read object from filename using supplied custom reader and item id"""
+        quietprint('[viset.cache.get][HIT]: Reading "' + reader + '" encoded object with id "' + str(id) + '" from file "' + filename + '"', self._verbose)
+        if reader == 'h5py':
+            f = h5py.File(filename,'r')
+            obj = f[id]  # FIXME: this is a link to an hdf5 object and not a numpy object.  Deep copy?
+            return obj
+        elif reader == 'universal':
             return pylab.imread(filename)
         elif reader == 'mnist':
             return viset.library.mnist.imread(filename, int(id))
         else:
-            self.quietprint('[viset.cache][ERROR]: reader type "' + str(reader) + '" unsupported', self._verbose)
+            self.quietprint('[viset.cache.read][ERROR]: reader type "' + str(reader) + '" unsupported', self._verbose)
             raise NotImplementedError()
         
     def unpack(self, filename, sha1=None, item=None):
@@ -83,25 +91,28 @@ class Cache():
     def load(self, filename, reader=None, id=None):
         """Load a filename to an object"""
         if isimg(filename):
+            # standard image formats
             obj = pylab.imread(filename)
         elif reader is not None:
-            obj = self.read(filename, reader=reader, id=id)
+            # custom reader defined in querystring
+            obj = self.read(filename, reader=reader, id=id)  
         elif path.isdir(viset.util.splitextension(filename)[0]):
-            obj = viset.util.splitextension(filename)[0]  # archive directory in cache
+            # archive directory in cache
+            obj = viset.util.splitextension(filename)[0] 
         elif isarchive(filename):
-            quietprint('[viset.cache][ERROR]: archive file "' + filename + '" not unpacked to "' + (viset.util.splitextension(filename)[0]), self._verbose)
+            quietprint('[viset.cache.load][ERROR]: archive file "' + filename + '" not unpacked to "' + (viset.util.splitextension(filename)[0]), self._verbose)
             raise IOError()
         elif ishdf5(filename):
             obj = filename
         elif isfile(filename):
             obj = filename
         else:
-            quietprint('[viset.cache][ERROR]: could not load filename "' + filename + '"', self._verbose)
+            quietprint('[viset.cache.load][ERROR]: could not load filename "' + filename + '"', self._verbose)
             raise IOError()
         return obj
         
     def download(self, url):        
-        """Download element according to URL scheme"""
+        """Download url to cached file according to URL scheme"""
         self.free()  # garbage collection time?        
         filename = self.cachefile(urlparse.urldefrag(url)[0])
         url_scheme = urlparse.urlparse(url)[0]
@@ -124,11 +135,11 @@ class Cache():
         os.makedirs(self._cacheroot)        
         
     def fetch(self, url, sha1=None):
-        """Fetch and element and store in cache"""
+        """Fetch a url and unpack in cache"""
         urlopts = self._url_fragment_options(url, sha1)
         if isurl(url):            
             if not self.iscached(url):
-                quietprint('[viset.cache][MISS]: downloading \'' + str(url[0:63] + '...') + '\'', self._verbose)                                        
+                quietprint('[viset.cache][MISS]: downloading "' + str(url[0:63] + '...') + '"', self._verbose)                                        
                 filename = self.download(url)
                 if isarchive(filename):
                     quietprint('[viset.cache][MISS]: extracting archive \'' + filename + '\'', self._verbose)
@@ -138,54 +149,63 @@ class Cache():
                         filename = self.unpack(filename, sha1=urlopts['sha1'], item=None)                        
             else:
                 filename = self.cachefile(url)            
-                quietprint('[viset.cache][HIT]: retrieving \'' + filename + '\'', self._verbose)                                                     
+                quietprint('[viset.cache.get][HIT]: fetching "' + url + '" from "' + filename + '"', self._verbose)                                                     
         elif viset.util.isfile(self.cachefile(url)):            
             filename = self.cachefile(url)            
-            quietprint('[viset.cache][HIT]: retrieving \'' + filename + '\'', self._verbose)                                     
+            quietprint('[viset.cache.get][HIT]: fetching "' + url + '" from "' + filename + '"', self._verbose)                                                                 
         elif viset.util.isfile(self.abspath(path.basename(url))):
             filename = self.abspath(path.basename(url))
-            quietprint('[viset.cache][HIT]: retrieving \'' + filename + '\'', self._verbose)                                     
+            quietprint('[viset.cache.get][HIT]: fetching "' + url + '" from "' + filename + '"', self._verbose)                                                                             
         else:
             # nothing in cache or backing store
-            quietprint('[viset.cache][MISS]: \'' + url + '\' not available in cache', self._verbose)                                     
+            quietprint('[viset.cache.get][MISS]: "' + url + '" not available in cache', self._verbose)                                     
             return None
         
-        quietprint('[viset.cache][HIT]: loading \'' + filename + '\'', self._verbose)                                         
+        quietprint('[viset.cache.get][HIT]: loading "' + filename + '"', self._verbose)                                         
         return self.load(filename, reader=urlopts['reader'], id=urlopts['id'])
 
-    def put(self, obj):
-        """Put a numpy object into cache and return cached file""" 
+    def put(self, obj, async=False, key='obj', filename=None):
+        """Put a numpy object into cache and return url""" 
         if 'numpy' not in str(type(obj)):
             if type(obj) in [list, tuple]:
                 obj = numpy.array(obj)  # type coersion
             else:
                 raise ValueError('numpy object required for caching')                    
-        byteview = obj.view(numpy.uint8)
-        hash = hashlib.sha1(byteview).hexdigest() # byte view sha1
-        filename = self.abspath(hash + '.h5')
-        f = h5py.File(filename, 'w')
-        f['obj'] = obj
-        f.close()
-        return filename
+        if filename is None:
+            byteview = obj.view(numpy.uint8)
+            filename = hashlib.sha1(byteview).hexdigest() # byte view sha1
+        url = 'file://' + self.abspath(filename + '.h5') + '#reader=h5py&id=' + key
+        if self.iscached(url):
+            quietprint('[viset.cache.put][HIT]: "' + self.cachefile(url) + '"', self._verbose)                                                     
+        else:
+            quietprint('[viset.cache.put][MISS]: writing "' + url + '"', self._verbose)                                                                 
+            filename = self.cachefile(url)
+            f = h5py.File(filename, 'a')
+            f[key] = obj
+            f.close()
+        return CachedObject(url) if async else url
         
-    def get(self, url, sha1=None):
-        """Get an object from cache""" 
-        try:
-            return self.fetch(url, sha1=sha1)
-        except KeyboardInterrupt:
-            pass
-        except NotImplementedError:
-            raise
-        except:
-            if self._refetch:
-                quietprint('[viset.cache][ERROR]: cache fetch error - discarding and refetching', self._verbose)                                                     
-                self.discard(url)
+    def get(self, url, sha1=None, async=False):
+        """Get an url from cache and return object""" 
+        if async:
+            return CachedResult(url, sha1=sha1, verbose=self._verbose, cache=self)
+        else:
+            try:
                 return self.fetch(url, sha1=sha1)
-            else:
+            except KeyboardInterrupt:
+                pass
+            except NotImplementedError:
                 raise
+            except:
+                if self._refetch:
+                    quietprint('[viset.cache][ERROR]: cache fetch error - discarding and refetching', self._verbose)                                                     
+                    self.discard(url)
+                    return self.fetch(url, sha1=sha1)
+                else:
+                    raise
             
     def discard(self, url):
-        """Delete element from cache"""
+        """Delete url from cache"""
         if path.isfile(self.cachefile(url)):
             os.remove(self.cachefile(url))
         if path.isdir(self.cachedir(url)):
@@ -203,7 +223,7 @@ class Cache():
         return total_size
 
     def hash(self, url, prettyhash=_prettyhash):
-        """Compute a SHA1 file hash to uniquely name a cached file"""
+        """Compute a SHA1 hash of a url to generate a unique cache filename"""
         p = urlparse.urlsplit(url)
         urlquery = urlparse.urlunsplit([p[0],p[1],p[2],p[3],None])
         urlpath = urlparse.urlunsplit([p[0],p[1],p[2],None,None])        
@@ -227,7 +247,7 @@ class Cache():
         self._free_ctr -= 1
     
     def cachefile(self, url):
-        """The file name of the element in the cache"""
+        """The file name of the url in the cache"""
         p = urlparse.urlsplit(url)
         urlquery = urlparse.urlunsplit([p[0],p[1],p[2],p[3],None])        
         urlpath = urlparse.urlunsplit([p[0],p[1],p[2],None,None])
@@ -240,19 +260,19 @@ class Cache():
             return self.abspath(str(urlhash) + str(ext))
 
     def cachedir(self, archive_url):
-        """The unpacked directory name of archive files in the cache"""
+        """The unpacked directory name of archive file at url in the cache"""
         p = urlparse.urlsplit(archive_url)
         urlpath = urlparse.urlunsplit([p[0],p[1],p[2],p[3],None])
         urlopt = self._url_fragment_options(archive_url)
         urlhash = self.hash(urlpath)
         return self.abspath(urlhash)
         
-    def abspath(self, filename):
-        """The absolute file path for elements in the cache"""
-        return path.join(self._cacheroot, filename)
+    def abspath(self, basename):
+        """The absolute file path for a basename in the cache"""
+        return path.join(self._cacheroot, basename)
         
     def iscached(self, url):
-        """Return true if an element is in the cache"""
+        """Return true if a url is in the cache"""
         return path.isfile(self.cachefile(url)) or path.isdir(self.cachedir(url))
             
             
@@ -261,20 +281,22 @@ def quietprint(mystr, is_verbose):
         print mystr
             
 
-class CachedResult(object):
+class CachedObject(object):
     """Mirrors Python's AsyncResult class"""
     _obj = None    
     _url = None
+    _sha1 = None
     _verbose = False
     _cache = None
 
-    def __init__(self, url, verbose=True, cache=None):
+    def __init__(self, url, sha1=None, verbose=True, cache=None):
         self._verbose = verbose
         if cache is None:
             self._cache = Cache(verbose=self._verbose)
         else:
             self._cache = cache
         self._url = url
+        self._sha1 = sha1        
     
     def successful():
         return (self._obj is not None)
@@ -284,15 +306,17 @@ class CachedResult(object):
         return (self._obj is not None)
     
     def get(self):
-        self._obj = self._cache.get(self._url)
+        if self._obj is None:
+            self._obj = self._cache.get(self._url, sha1=self._sha1, async=False)
         return self._obj
         
     def __repr__(self):
         return str('<viset.cache: obj=' + str(type(self._obj)) + ', cached=' + str(self._cache.iscached(self._url)) + ', URL=\'' + str(self._url) + '\'>')        
     
-
-class CachedImage(CachedResult):
+class CachedImage(CachedObject):
     def __repr__(self):
         return str('<viset.image: obj=' + str(type(self._obj)) + ', cached=' + str(self._cache.iscached(self._url)) + ', URL=\'' + str(self._url) + '\'>')        
 
     
+class CachedResult(CachedObject):
+    pass
