@@ -42,7 +42,7 @@ def download(url, output_filename, sha1=None, verbose=True, md5=None):
 
     # socket.setdefaulttimeout(10)  # do not set globally
         
-    page = urlopen(url)
+    page = urlopen(url, timeout=10)
     page_info = page.info()
 
     output_file = open(output_filename, 'wb+')
