@@ -8,6 +8,13 @@ import inspect
 import IPython
 import pdb
 import datetime
+from time import gmtime, strftime, localtime
+
+
+def timestamp():
+    """Return date and time string in form DDMMMYY_HHMMSS"""
+    return string.upper(strftime("%d%b%y_%I%M%S%p", localtime()))
+
 
 def isurl(path):
     return urlparse.urlparse(path).scheme != ""
